import processing.serial.*;
import java.awt.Color;

DioderDriver driver;

// UI controls
List<Control> controls;
Wheeler wheelerRed, wheelerGreen, wheelerBlue;
Wheeler wheelerH, wheelerS, wheelerB;
Wheeler wheelerGammaR, wheelerGammaG, wheelerGammaB;
ColorView viewIn, viewOut;
Plot plotMapR, plotMapG, plotMapB;

void setup() {
  size(768, 384);
  driver = new DioderDriver(null);
  driver.mapR = driver.gammaCorrection(2.2);
  
  createUI();
  
  addMouseWheelListener(new java.awt.event.MouseWheelListener() { 
    public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) { 
      mouseWheel(evt.getWheelRotation());
  }});
}

void draw() {
  for (Control control : controls) {
    control.draw();  
  }
  
  driver.r = (char) wheelerRed.value;
  driver.g = (char) wheelerGreen.value;
  driver.b = (char) wheelerBlue.value;
  
  viewIn.r = (int) wheelerRed.value;
  viewIn.g = (int) wheelerGreen.value;
  viewIn.b = (int) wheelerBlue.value;
  
  viewOut.r = driver.mapR[(int) wheelerRed.value];
  viewOut.g = driver.mapG[(int) wheelerGreen.value];
  viewOut.b = driver.mapB[(int) wheelerBlue.value];  
  
  driver.update();
}

void mouseWheel(int delta) {
  for (Control control : controls) {
    if (control.contains(mouseX,mouseY)) {
      control.mouseWheel(delta);
      if (control == wheelerGammaR) {
        plotMapR.data = driver.mapR = driver.gammaCorrection(wheelerGammaR.value);
      }
      else if (control == wheelerGammaG) {
        plotMapG.data = driver.mapG = driver.gammaCorrection(wheelerGammaG.value);
      }
      else if (control == wheelerGammaB) {
        plotMapB.data = driver.mapB = driver.gammaCorrection(wheelerGammaB.value);
      }      
      
      else if (control == wheelerH || control == wheelerS || control == wheelerB) {
        color c = Color.HSBtoRGB(wheelerH.value / 255.0f, wheelerS.value / 255.0f, wheelerB.value / 255.0f);
        wheelerRed.value = red(c);
        wheelerGreen.value = green(c);
        wheelerBlue.value = blue(c);
      }
      
      else if (control == wheelerRed || control == wheelerGreen || control == wheelerBlue) {
        float[] arrHsb = Color.RGBtoHSB((int)wheelerRed.value, (int)wheelerGreen.value, (int)wheelerBlue.value, null);
        wheelerH.value = arrHsb[0] * 255;
        wheelerS.value = arrHsb[1] * 255;
        wheelerB.value = arrHsb[2] * 255;
      }      
      

    }
  }
}

void createUI () {
  controls = new LinkedList<Control>();
  
  wheelerRed = new Wheeler(0,0,256,32,"red");
  wheelerGreen = new Wheeler(0,32,256,32,"green");
  wheelerBlue = new Wheeler(0,64,256,32,"blue");
  
  wheelerH = new Wheeler(512,0,256,32,"hue");
  controls.add(wheelerH);
  wheelerS = new Wheeler(512,32,256,32,"saturation");
  controls.add(wheelerS);
  wheelerB = new Wheeler(512,64,256,32,"brightness");  
  controls.add(wheelerB);
  
  wheelerGammaR = new Wheeler(0,352,256,32,"gamma(R)");
  wheelerGammaR.value = 2.2;
  wheelerGammaR.step = 0.05;
  wheelerGammaR.valMin = 0;
  wheelerGammaR.valMax = 10;
  controls.add(wheelerGammaR);
  
  wheelerGammaG = new Wheeler(256,352,256,32,"gamma(G)");
  wheelerGammaG.value = 2.2;
  wheelerGammaG.step = 0.05;
  wheelerGammaG.valMin = 0;
  wheelerGammaG.valMax = 10;  
  controls.add(wheelerGammaG);

  wheelerGammaB = new Wheeler(512,352,256,32,"gamma(B)");
  wheelerGammaB.value = 2.2;
  wheelerGammaB.step = 0.05;
  wheelerGammaB.valMin = 0;
  wheelerGammaB.valMax = 10;  
  controls.add(wheelerGammaB);
  
  plotMapR = new Plot(0,96,256,256,"mapR");
  plotMapR.col = color(255,0,0);
  plotMapR.data = driver.mapR;
  
  plotMapG = new Plot(256,96,256,256,"mapG");
  plotMapG.col = color(0,255,0);
  plotMapG.data = driver.mapG;

  plotMapB = new Plot(512,96,256,256,"mapB");
  plotMapB.col = color(0,0,255);
  plotMapB.data = driver.mapB;  

  viewIn = new ColorView(256,0,256,48,"RGB ");
  viewOut = new ColorView(256,48,256,48,"RGB corrected");
  
  plotMapR.data = driver.mapR = driver.gammaCorrection(wheelerGammaR.value);
  plotMapG.data = driver.mapG = driver.gammaCorrection(wheelerGammaG.value);
  plotMapB.data = driver.mapB = driver.gammaCorrection(wheelerGammaB.value);
  
  controls.add(wheelerRed);
  controls.add(wheelerGreen);
  controls.add(wheelerBlue);
  controls.add(viewIn);
  controls.add(viewOut);
  controls.add(plotMapR);
  controls.add(plotMapG);
  controls.add(plotMapB);  
}

